@@ -82,9 +82,7 @@ class Expr:
         return self
 
     def _operands_for_repr(self):
-        return [
-            f"{param}={repr(op)}" for param, op in zip(self._parameters, self.operands)
-        ]
+        return [f"{param}={op!r}" for param, op in zip(self._parameters, self.operands)]
 
     def __str__(self):
         s = ", ".join(self._operands_for_repr())
@@ -103,7 +101,7 @@ class Expr:
 
         if repr(op) != repr(default):
             if param:
-                header += f" {param}={repr(op)}"
+                header += f" {param}={op!r}"
             else:
                 header += repr(op)
         return header
@@ -543,7 +541,7 @@ class Expr:
 
     @functools.cached_property
     def _name(self) -> str:
-        return self._funcname + "-" + self.deterministic_token
+        return f"{self._funcname}-{self.deterministic_token}"
 
     @property
     def _meta(self):
@@ -827,7 +825,7 @@ class Expr:
         assert (
             isinstance(operation, tuple)
             and all(issubclass(e, Expr) for e in operation)
-            or issubclass(operation, Expr)  # type: ignore
+            or issubclass(operation, Expr)  # type: ignore[arg-type]
         ), "`operation` must be`Expr` subclass)"
         return (expr for expr in self.walk() if isinstance(expr, operation))
 
@@ -936,7 +934,8 @@ def optimize_until(expr: Expr, stage: OptimizerStage) -> Expr:
         return expr
 
     # Manipulate Expression to make it more efficient
-    expr = expr.rewrite(kind="tune", rewritten={})
+    if dask.config.get("optimization.tune.active", True):
+        expr = expr.rewrite(kind="tune", rewritten={})
     if stage == "tuned-logical":
         return expr
 
