@@ -357,7 +357,7 @@ def reshape(x, shape, merge_chunks=True, limit=None):
     din = len(x.shape)
     dout = len(shape)
     if not merge_chunks and din > dout:
-        x = x.rechunk({i: 1 for i in range(din - dout)})
+        x = x.rechunk(dict.fromkeys(range(din - dout), 1))
 
     inchunks, outchunks, _, _ = reshape_rechunk(x.shape, shape, x.chunks)
     x2 = x.rechunk(inchunks)
@@ -378,8 +378,8 @@ def reshape(x, shape, merge_chunks=True, limit=None):
 def _sanity_checks(x, shape):
     if np.isnan(sum(x.shape)):
         raise ValueError(
-            "Array chunk size or shape is unknown. shape: %s\n\n"
-            "Possible solution with x.compute_chunk_sizes()" % str(x.shape)
+            f"Array chunk size or shape is unknown. shape: {x.shape}\n\n"
+            "Possible solution with x.compute_chunk_sizes()"
         )
 
     if reduce(mul, shape, 1) != x.size:
