@@ -4,6 +4,7 @@ import itertools
 import pickle
 import sys
 from collections import namedtuple
+from collections.abc import Mapping
 
 import pytest
 
@@ -11,17 +12,30 @@ from dask._task_spec import (
     Alias,
     DataNode,
     DependenciesMapping,
+    Dict,
+    List,
+    Set,
     Task,
     TaskRef,
+    Tuple,
     _get_dependencies,
     convert_legacy_graph,
     execute_graph,
+    fuse_linear_task_spec,
+    parse_input,
     resolve_aliases,
 )
-from dask.base import tokenize
 from dask.core import keys_in_tasks, reverse_dict
-from dask.optimization import SubgraphCallable
 from dask.sizeof import sizeof
+from dask.tokenize import tokenize
+from dask.utils import funcname
+
+
+def convert_and_verify_keys(dsk):
+    new_dsk = convert_legacy_graph(dsk)
+    vals = [(k, v.key) for k, v in new_dsk.items()]
+    assert all(v[0] == v[1] for v in vals), vals
+    return new_dsk
 
 
 def identity(x):
@@ -47,7 +61,7 @@ def test_convert_legacy_dsk_skip_new():
     dsk = {
         "key-1": Task("key-1", func, "a", "b"),
     }
-    converted = convert_legacy_graph(dsk)
+    converted = convert_and_verify_keys(dsk)
     assert converted["key-1"] is dsk["key-1"]
     assert converted == dsk
 
@@ -57,9 +71,7 @@ def test_repr():
     assert repr(t) == "<Task 'key' func('a', 'b')>"
 
     t = Task("nested", func2, t, t.ref())
-    assert (
-        repr(t) == "<Task 'nested' func2(<Task 'key' func('a', 'b')>, Alias(key->key))>"
-    )
+    assert repr(t) == "<Task 'nested' func2(<Task 'key' func('a', 'b')>, Alias('key'))>"
 
     def long_function_name_longer_even_longer(a, b):
         return a + b
@@ -74,6 +86,33 @@ def test_repr():
     assert repr(t) == "<Task 'kwarg' use_kwargs('foo', kwarg='kwarg_value')>"
 
 
+def test_task_eq():
+    assert Alias(("x", 0, 0)) == Alias(("x", 0, 0))
+    assert Alias(("x", 0, 0)) != Alias(("x", 0, 1))
+
+    assert List(Alias(("x", 0, 0))) == List(Alias(("x", 0, 0)))
+    assert List(Alias(("x", 0, 0))) == List(Alias("a", "a")).substitute(
+        {"a": ("x", 0, 0)}
+    )
+    assert List(Alias(("x", 0, 0)), Alias(("x", 0, 1))) != List(
+        Alias(("x", 0, 0)), Alias(("x", 0, 2))
+    )
+
+    obj = Task(
+        ("z", 0, 0),
+        func,
+        List(TaskRef(("x", 0, 0)), TaskRef(("x", 0, 1))),
+        List(TaskRef(("y", 0, 0)), TaskRef(("y", 1, 0))),
+    )
+    assert obj == obj
+    assert obj == Task(
+        ("z", 0, 0),
+        func,
+        List(TaskRef(("x", 0, 0)), TaskRef(("x", 0, 1))),
+        List(TaskRef(("y", 0, 0)), TaskRef(("y", 1, 0))),
+    )
+
+
 def long_function_name_longer_even_longer(a, b):
     return a + b
 
@@ -82,31 +121,6 @@ def use_kwargs(a, kwarg=None):
     return a + kwarg
 
 
-def test_unpickled_repr():
-    t = pickle.loads(pickle.dumps(Task("key", func, "a", "b")))
-    assert repr(t) == "<Task 'key' func('a', 'b')>"
-
-    t = pickle.loads(pickle.dumps(Task("nested", func2, t, t.ref())))
-    assert (
-        repr(t) == "<Task 'nested' func2(<Task 'key' func('a', 'b')>, Alias(key->key))>"
-    )
-
-    t = pickle.loads(
-        pickle.dumps(Task("long", long_function_name_longer_even_longer, t, t.ref()))
-    )
-    assert repr(t) == "<Task 'long' long_function_name_longer_even_longer(...)>"
-
-    t = pickle.loads(
-        pickle.dumps(Task("kwarg", use_kwargs, "foo", kwarg="kwarg_value"))
-    )
-    assert repr(t) == "<Task 'kwarg' use_kwargs('foo', kwarg='kwarg_value')>"
-
-
-def _assert_dsk_conversion(new_dsk):
-    vals = [(k, v.key) for k, v in new_dsk.items()]
-    assert all(v[0] == v[1] for v in vals), vals
-
-
 def test_convert_legacy_dsk():
     def func(*args):
         return "-".join(args)
@@ -126,8 +140,7 @@ def test_convert_legacy_dsk():
             (func, "const", "bar"),
         ],
     }
-    new_dsk = convert_legacy_graph(dsk)
-    _assert_dsk_conversion(new_dsk)
+    new_dsk = convert_and_verify_keys(dsk)
     t1 = new_dsk["key-1"]
     assert isinstance(t1, Task)
     assert t1.func == func
@@ -168,17 +181,19 @@ def test_task_executable():
 
 
 def test_task_nested_sequence():
+    # We do **not** recurse into structures since this kills performance
+    # we're only considering top-level task objects as arguments or kwargs
     t1 = Task("key-1", func, "a", "b")
     t2 = Task("key-2", func2, "c", "d")
 
     tseq = Task("seq", identity, [t1, t2])
-    assert tseq() == [func("a", "b"), func2("c", "d")]
+    assert tseq() == [t1, t2]
 
     tseq = Task("set", identity, {t1, t2})
-    assert tseq() == {func("a", "b"), func2("c", "d")}
+    assert tseq() == {t1, t2}
 
     tseq = Task("dict", identity, {"a": t1, "b": t2})
-    assert tseq() == {"a": func("a", "b"), "b": func2("c", "d")}
+    assert tseq() == {"a": t1, "b": t2}
 
 
 def test_reference_remote():
@@ -233,16 +248,87 @@ class SerializeOnlyOnce:
 
 def test_pickle():
 
+    def assert_slots_equal(a, b):
+        def get_all_slots(obj):
+            slots = set()
+            for cls in obj.__class__.mro():
+                slots.update(getattr(cls, "__slots__", ()))
+            return slots
+
+        all_slots = get_all_slots(a) | get_all_slots(a)
+        assert all_slots == get_all_slots(a) == get_all_slots(a)
+        assert all(getattr(a, slot) == getattr(b, slot) for slot in all_slots)
+        assert not hasattr(a, "__dict__")
+        assert not hasattr(b, "__dict__")
+
     t1 = Task("key-1", func, "a", "b")
     t2 = Task("key-2", func, "c", "d")
 
     rtt1 = pickle.loads(pickle.dumps(t1))
+    assert repr(rtt1) == repr(t1)
     rtt2 = pickle.loads(pickle.dumps(t2))
+
+    assert_slots_equal(t1, rtt1)
+    assert_slots_equal(t2, rtt2)
     assert t1 == rtt1
     assert t1.func == rtt1.func
     assert t1.func is rtt1.func
     assert t1.func is rtt2.func
 
+    l = Tuple(t1, t2)
+    rtl = pickle.loads(pickle.dumps(l))
+    assert l == rtl
+    assert l() == rtl()
+    d = Dict(key=t1)
+    rtd = pickle.loads(pickle.dumps(d))
+    assert d == rtd
+    assert d() == rtd()
+
+
+def test_pickle_size():
+    # We will serialize many of these objects which drives both memory usage and
+    # serialization runtime performance.
+    # Reducing pickle size is beneficial but the numbers below are determined
+    # empirically
+    # Analyzing the output with pickletools.dis is useful to debug memoization
+    # and serialization by value
+
+    a = Alias("a", "b")
+    # We cannot shrink it to nothing
+    assert len(pickle.dumps(a)) < 55
+    b = Alias("b", "c")
+    # But most of it should be overhead that is memoized
+    assert len(pickle.dumps((a, b))) <= 70
+
+    # Pickle should be able to memoize this. On py3.10 that's 2 additional bytes
+    assert len(pickle.dumps((a, b, b))) <= len(pickle.dumps((a, b))) + 10
+
+    t1 = Task("key-1", func, "a", "b")
+    assert len(pickle.dumps(t1)) < 120
+
+    t2 = Task("key-2", func, TaskRef("key-1"), "c")
+    assert len(pickle.dumps(t2)) < 140
+
+    assert len(pickle.dumps((t1, t2))) < 170
+
+    l = List(t1, t2)
+    assert len(pickle.dumps(l)) <= 272
+
+    sizes = []
+    growth = []
+    inner = List(t1, t2)
+    for depth in range(20):
+        inner = List(inner, t1)
+        size = len(pickle.dumps(inner))
+        if len(sizes) > 0:
+            growth.append(size - sizes[-1][1])
+        sizes.append((depth, size))
+    growth = set(growth)
+    # If this breaks, something cannot be memoized. That's very concerning
+    assert len(growth) == 1
+    # If this goes up, that's not great but not a disaster
+    assert growth.pop() <= 32
+
 
 def test_tokenize():
     t = Task("key-1", func, "a", "b")
@@ -288,8 +374,7 @@ def test_parse_curry():
     dsk = {
         "key-1": (curry, func, "a", "b"),
     }
-    converted = convert_legacy_graph(dsk)
-    _assert_dsk_conversion(converted)
+    converted = convert_and_verify_keys(dsk)
     t = Task("key-1", curry, func, "a", "b")
     assert converted["key-1"]() == t()
     assert t() == "a-bc"
@@ -310,7 +395,7 @@ def test_avoid_cycles():
     dsk = {
         "key": TaskRef("key"),  # e.g. a persisted key
     }
-    new_dsk = convert_legacy_graph(dsk)
+    new_dsk = convert_and_verify_keys(dsk)
     assert not new_dsk
 
 
@@ -360,102 +445,6 @@ def test_array_as_argument():
     assert not _get_dependencies(np.array([1, t.ref()]))
 
 
-def test_subgraph_callable():
-    def add(a, b):
-        return a + b
-
-    subgraph = SubgraphCallable(
-        {
-            "_2": (add, "_0", "_1"),
-            "_3": (add, TaskRef("add-123"), "_2"),
-        },
-        "_3",
-        ("_0", "_1"),
-    )
-
-    dsk = {
-        "a": 1,
-        "b": 2,
-        "c": (subgraph, "a", "b"),
-    }
-    new_dsk = convert_legacy_graph(dsk)
-    _assert_dsk_conversion(new_dsk)
-    assert new_dsk["c"].dependencies == {"a", "b", "add-123"}
-    assert (
-        new_dsk["c"](
-            {
-                "add-123": 123,
-                "a": 1,
-                "b": 2,
-            }
-        )
-        == 126
-    )
-
-
-def apply(func, args, kwargs=None):
-    return func(*args, **(kwargs or {}))
-
-
-def test_redirect_to_subgraph_callable():
-    subgraph = SubgraphCallable(
-        {
-            "out": (apply, func3, ["_0"], (dict, [["some", "dict"]])),
-        },
-        "out",
-        ("_0",),
-    )
-    dsk = {
-        "alias": "foo",
-        "foo": (func, (subgraph, (func2, "bar", "baz")), "second_arg"),
-    }
-    new_dsk = convert_legacy_graph(dsk)
-    from dask import get
-
-    expected = get(dsk, ["foo"])[0]
-    assert expected == "bar=baz//some=dict-second_arg"
-    t = new_dsk["alias"]
-    assert t.dependencies == {"foo"}
-    t = new_dsk["foo"]
-    assert not t.dependencies
-
-    assert t() == expected
-
-
-def test_inline():
-    t = Task("key-1", func, "a", TaskRef("b"))
-
-    assert t.inline({"b": DataNode(None, "b")})() == "a-b"
-
-    assert t.inline({"b": Task(None, func, "b", "c")})() == "a-b-c"
-
-    t2 = Task("key-1", func, TaskRef("a"), TaskRef("b"))
-    inline2 = t2.inline({"a": DataNode(None, "a")})
-    assert inline2.dependencies == {"b"}
-
-    assert inline2({"b": "foo"}) == "a-foo"
-
-    t = Task("foo", identity, [t, t2])
-    assert t.dependencies == {"b", "a"}
-    inlined = t.inline({"a": DataNode(None, "foo")})
-    assert inlined.dependencies == {"b"}
-    assert inlined({"b": "bar"}) == ["a-bar", "foo-bar"]
-
-    t = Task("key-2", identity, [t, t2])
-    t = Task("key-2", identity, {"foo": t, "bar": t2})
-    inlined = t.inline(
-        {
-            "a": DataNode(None, "foo"),
-            "bar": DataNode(None, "bar"),
-        }
-    )
-    assert inlined({"b": "bar"})
-
-    t = Task("key-1", identity, [TaskRef("a"), TaskRef("b")])
-    inlined = t.inline({"a": DataNode(None, "a"), "b": DataNode(None, "b")})
-    assert inlined() == ["a", "b"]
-
-
 @pytest.mark.parametrize(
     "inst",
     [
@@ -505,8 +494,7 @@ def test_parse_graph_namedtuple_legacy(typ, args, kwargs):
         return x
 
     dsk = {"foo": (func, typ(*args, **kwargs))}
-    new_dsk = convert_legacy_graph(dsk)
-    _assert_dsk_conversion(new_dsk)
+    new_dsk = convert_and_verify_keys(dsk)
 
     assert new_dsk["foo"]() == typ(*args, **kwargs)
 
@@ -524,7 +512,7 @@ def test_parse_namedtuple(typ, args, kwargs):
         return x
 
     obj = typ(*args, **kwargs)
-    t = Task("foo", func, obj)
+    t = Task("foo", func, parse_input(obj))
 
     assert t() == obj
 
@@ -532,7 +520,7 @@ def test_parse_namedtuple(typ, args, kwargs):
     if typ is PlainNamedTuple:
         args = tuple([TaskRef("b")] + list(args)[1:])
         obj = typ(*args, **kwargs)
-        t = Task("foo", func, obj)
+        t = Task("foo", func, parse_input(obj))
         assert t.dependencies == {"b"}
         assert t({"b": "foo"}) == typ(*tuple(["foo"] + list(args)[1:]), **kwargs)
 
@@ -544,6 +532,11 @@ def test_pickle_literals():
     assert roundtripped == obj
 
 
+@pytest.mark.parametrize("obj", [set, {0}, [], [1], {}, {2: 3}, (), (4,)])
+def test_parse_non_task_inputs(obj):
+    assert parse_input(obj) == obj
+
+
 def test_resolve_aliases():
     tasks = [
         Alias("bar", "foo"),
@@ -630,7 +623,7 @@ def test_convert_resolve():
         "fourth": "third",
         "fifth": (func, "fourth"),
     }
-    dsk = convert_legacy_graph(dsk)
+    dsk = convert_and_verify_keys(dsk)
     assert len(dsk) == 5
 
     optimized = resolve_aliases(dsk, {"fifth"}, reverse_dict(DependenciesMapping(dsk)))
@@ -865,39 +858,288 @@ def test_fused_dont_hold_in_memory_too_long():
     assert OnlyTwice.total == 10
 
 
-def test_subgraph_dont_hold_in_memory_too_long_legacy():
-    prev = None
-
-    # If we execute a fused task we want to release objects as quickly as
-    # possible. If every task generates this object, we must at most hold two of
-    # them in memory
-    class OnlyTwice:
-        counter = 0
-        total = 0
-
-        def __init__(self):
-            OnlyTwice.counter += 1
-            OnlyTwice.total += 1
-            if OnlyTwice.counter > 2:
-                raise ValueError("Didn't release as expected")
-
-        def __del__(self):
-            OnlyTwice.counter -= 1
-
-    def generate_object(arg):
-        return OnlyTwice()
-
-    prev = None
-    subgraph = {}
-    for ix in range(10):
-        subgraph[f"key-{ix}"] = (generate_object, prev if prev else "foo")
-        prev = f"key-{ix}"
-    subgraph_callable = SubgraphCallable(
-        subgraph,
-        "key-9",
-        (),
+def test_linear_fusion():
+    tasks = [
+        io := Task("foo", func, 1),
+        second := Task("second", func, io.ref()),
+        Task("third", func, second.ref()),
+    ]
+    dsk = {t.key: t for t in tasks}
+    result = fuse_linear_task_spec(dsk, {"third"})
+    assert len(result) == 2
+    assert isinstance(result["third"], Alias)
+    assert (
+        isinstance(result["foo-second-third"], Task)
+        and funcname(result["foo-second-third"].func) == "_execute_subgraph"
     )
-    dsk = {"bar": (subgraph_callable,)}
-    converted = convert_legacy_graph(dsk)
-    assert converted["bar"]()
-    assert OnlyTwice.total == 10
+
+    # Data Nodes don't get fused
+
+    tasks = [
+        io := DataNode("foo", 1),
+        second := Task("second", func, io.ref()),
+        third := Task("third", func, second.ref()),
+        Task("fourth", func, third.ref()),
+    ]
+    dsk = {t.key: t for t in tasks}
+    result = fuse_linear_task_spec(dsk, {"fourth"})
+    assert len(result) == 2
+    assert isinstance(result["fourth"], Alias)
+    assert (
+        isinstance(result["foo-second-third-fourth"], Task)
+        and funcname(result["foo-second-third-fourth"].func) == "_execute_subgraph"
+    )
+    assert "foo" not in result
+
+    # Branch, so no fusion
+    tasks.append(Task("branch", func, third.ref()))
+    dsk = {t.key: t for t in tasks}
+    result = fuse_linear_task_spec(dsk, {"fourth"})
+    assert len(result) == 4
+    assert "foo-second-third" in result
+    assert isinstance(result["third"], Alias)
+
+    # Branch, so no fusion at all
+    tasks.append(Task("branch2", func, second.ref()))
+    dsk = {t.key: t for t in tasks}
+    result = fuse_linear_task_spec(dsk, {"fourth"})
+    assert len(result) == 6
+    assert not any("-" in k for k in dsk)
+
+
+def test_linear_fusion_intermediate_branch():
+    tasks = [
+        io := DataNode("foo", 1),
+        second := Task("second", func, io.ref()),
+        third := Task("third", func, second.ref()),
+        other := Task("other", func, 1),
+        Task("fourth", func, third.ref(), other.ref()),
+    ]
+    dsk = {t.key: t for t in tasks}
+    result = fuse_linear_task_spec(dsk, {"fourth"})
+    assert "foo-second-third" in result
+    assert isinstance(result["fourth"], Task)
+
+
+def test_linear_fusion_two_branches():
+    tasks = [
+        left_one := DataNode("left_one", "a"),
+        left_two := Task("left_two", func, left_one.ref()),
+        right_one := DataNode("right_one", "a"),
+        right_two := Task("right_two", func, right_one.ref()),
+        middle := Task("middle", func, right_two.ref(), left_two.ref()),
+        third := Task("third", func, middle.ref()),
+        Task("fourth", func, third.ref()),
+    ]
+    dsk = {t.key: t for t in tasks}
+    result = fuse_linear_task_spec(dsk, {"fourth"})
+    assert "left_one-left_two" in result
+    assert "right_one-right_two" in result
+    assert isinstance(result["right_two"], Alias)
+    assert isinstance(result["left_two"], Alias)
+    assert isinstance(result["fourth"], Alias)
+    assert isinstance(result["third-fourth"], Task)
+
+
+def test_linear_fusion_multiple_outputs():
+    tasks = [
+        first := DataNode("first", "a"),
+        second := Task("second", func, first.ref()),
+        third := Task("third", func, second.ref()),
+        Task("fourth", func, third.ref()),
+    ]
+    dsk = {t.key: t for t in tasks}
+    result = fuse_linear_task_spec(dsk, {"fourth", "second"})
+    assert "first-second" in result
+    assert "second" in result
+    assert isinstance(result["second"], Alias)
+    assert isinstance(result["fourth"], Alias)
+    assert isinstance(result["first-second"], Task)
+    assert isinstance(result["third-fourth"], Task)
+
+
+def test_nested_containers():
+    t = List(Task("key-1", func, "a", "b"), Task("key-2", func, "c", "d"))
+    assert t() == ["a-b", "c-d"]
+
+    t = List(Task("key-1", func, "a", TaskRef("b")), Task("key-2", func, "c", "d"))
+    assert t.dependencies == {"b"}
+    assert t({"b": "b"}) == ["a-b", "c-d"]
+
+
+def test_dict_class():
+    t = Dict(k=Task("key-1", func, "a", "b"), v=Task("key-2", func, "c", "d"))
+    assert not t.dependencies
+    assert t() == {"k": "a-b", "v": "c-d"}
+    t2 = Dict({"k": Task("key-1", func, "a", "b"), "v": Task("key-2", func, "c", "d")})
+    assert t == t2
+    assert t() == t2()
+    t = Dict(k=Task("key-1", func, "a", TaskRef("b")), v=Task("key-2", func, "c", "d"))
+    assert t.dependencies == {"b"}
+    assert t({"b": "b"}) == {"k": "a-b", "v": "c-d"}
+
+    t = Dict(
+        {
+            "k": Task("key-1", func, "a", TaskRef("b")),
+            # Use a hashable key that isn't a string. This cannot be used as
+            # kwargs, for instance
+            ("v", 1): Task("key-2", func, "c", "d"),
+        }
+    )
+    assert t.dependencies == {"b"}
+    assert t({"b": "b"}) == {"k": "a-b", ("v", 1): "c-d"}
+
+    t = Dict(
+        k=Task("key-1", func, "a", "b"),
+        v=Task("key-2", func, "c", "d"),
+    )
+    t2 = Dict(
+        v=Task("key-2", func, "c", "d"),
+        k=Task("key-1", func, "a", "b"),
+    )
+    assert t == t2
+    assert tokenize(t) == tokenize(t2)
+
+    d = Dict(
+        [
+            ["k", Task("key-1", func, "a", TaskRef("b"))],
+            ["v", Task("key-2", func, TaskRef("c"), "d")],
+        ]
+    )
+    assert d.dependencies == {"c", "b"}
+    assert d({"b": "b", "c": "c"}) == {"k": "a-b", "v": "c-d"}
+
+    d = Dict([("columns", ["a", "b"])])
+    assert d() == {"columns": ["a", "b"]}
+
+    d = Dict([["columns", ["a", "b"]]])
+    assert d() == {"columns": ["a", "b"]}
+    # Can be converted to a dict, e.g. also used as **kwargs
+    assert isinstance(t, Mapping)
+    assert dict(t) == {
+        "k": Task("key-1", func, "a", "b"),
+        "v": Task("key-2", func, "c", "d"),
+    }
+    assert len(t) == len(dict(t)) == 2
+
+    def as_kwargs(**kwargs):
+        return kwargs == {
+            "k": Task("key-1", func, "a", "b"),
+            "v": Task("key-2", func, "c", "d"),
+        }
+
+    assert as_kwargs(**t)
+
+
+def test_block_io_fusion():
+
+    class SubTask(Task):
+
+        @property
+        def block_fusion(self) -> bool:
+            return True
+
+    tasks = [
+        io := DataNode("foo", 1),
+        second := Task("second", func, io.ref()),
+        third := SubTask("third", func, second.ref()),
+        Task("fourth", func, third.ref()),
+    ]
+    dsk = {t.key: t for t in tasks}
+    result = fuse_linear_task_spec(dsk, {"fourth"})
+    assert "foo-second" in result
+    assert isinstance(result["fourth"], Task)
+    assert isinstance(result["third"], SubTask)
+
+
+def test_data_producer():
+    tasks = [
+        io := DataNode("foo", 1),
+        second := Task("second", func, io.ref(), _data_producer=True),
+        third := Task("third", func, io.ref(), _data_producer=True),
+        fourth := Task("fourth", func, second.ref()),
+        fifth := Task("fifth", func, third.ref()),
+        Task("sixth", func, fourth.ref(), fifth.ref()),
+    ]
+
+    dsk = {t.key: t for t in tasks}
+    result = fuse_linear_task_spec(dsk, {"fourth"})
+    assert "second-fourth" in result
+    assert "third-fifth" in result
+    assert result["second-fourth"].data_producer
+    assert result["third-fifth"].data_producer
+    assert not result["sixth"].data_producer
+    assert result["foo"].data_producer
+
+
+@pytest.mark.parametrize(
+    "task_type, python_type",
+    [
+        (Tuple, tuple),
+        (List, list),
+        (Set, set),
+    ],
+)
+def test_nested_containers_different_types(task_type, python_type):
+    t = task_type(Task("key-1", func, "a", TaskRef("b")), Task("key-2", func, "c", "d"))
+    assert t.dependencies == {"b"}
+    assert t({"b": "b"}) == python_type(("a-b", "c-d"))
+
+
+def test_substitute():
+    t1 = Task("key-1", func, TaskRef("a"), "b")
+    assert t1.substitute({"a": "a"}) is t1
+    assert t1.substitute({"a": "a"}, key="foo") is not t1
+    t2 = t1.substitute({"a": "c"})
+    assert t2 is not t1
+    assert t2.dependencies == {"c"}
+
+    assert t1({"a": "a"}) == t2({"c": "a"})
+
+
+def test_alias_task_ref_key():
+    t = Alias(TaskRef("a"), "b")
+    assert t.key == "a"
+    assert isinstance(t.key, str)
+
+
+def test_substitute_nested():
+    def func(alist):
+        return alist[0] + alist[1]["foo"]
+
+    t1 = Task(
+        "key-1",
+        func,
+        List(
+            TaskRef("a"),
+            Dict(
+                {
+                    "foo": TaskRef("b"),
+                }
+            ),
+        ),
+    )
+    assert t1.dependencies == {"a", "b"}
+    t2 = t1.substitute({"a": "c", "b": "d"})
+    assert t2.dependencies == {"c", "d"}
+    assert t1({"a": "a", "b": "b"}) == t2({"c": "a", "d": "b"})
+
+
+@pytest.mark.parametrize("Container", [Dict, List, Set, Tuple])
+def test_nested_containers_empty(Container):
+    assert Container(Container.klass())() == Container.klass()
+
+
+class MySubclass(Task):
+    __slots__ = ("custom_kwarg_only",)
+
+    def __init__(self, key, func, /, *args, custom_kwarg_only, **kwargs):
+        self.custom_kwarg_only = custom_kwarg_only
+        super().__init__(key, func, *args, **kwargs)
+
+
+def test_substitute_subclasses():
+    t = MySubclass("key", func, "a", TaskRef("b"), custom_kwarg_only="foo")
+    t2 = t.substitute({"b": "c"})
+    assert t2.custom_kwarg_only == "foo"
+    assert t2({"a": "a", "c": "b"}) == t({"a": "a", "b": "b"})
